package model.linearpub;

import java.util.List;
import java.util.function.Consumer;
import java.util.function.Function;

/**
 * Interface: QueryArrayIdea
 * 
 * 	Hello, I'm QueryArrayIdea
 * 
 *  I am a proud ADT (Java Interface)
 *  
 *  I offer many methods that allow convenient
 *  querying of an array.
 *  
 *  Once I'm constructed I am read-only. In other
 *  	words, I do [not] support adding/removing of elements
 *  
 *  Notes:
 *  	- Runtime exceptions should be type RuntimeException (unless otherwise noted)
 *  	- Implementors must code generics must be coded (as specified in this interface)
 *
 */

public interface QueryArrayIdea<E> {
	
	//------------------------------------------
	// Getting Statistics (Core)	

	/** Returns number of elements in this list (aka length/count) */
	int size();

	/** Return true is this list contains no elements */
	boolean isEmpty();
	
	//------------------------------------------
	// Accessing (Core)	

	/**
	 * Return element at passed zero-based index.
	 * If param "index" is invalid, throws an IndexOutOfBoundsException with this text: 
	 * 		"(get) index %d is out of bounds"
	 * where %d is replaced with the invalid index.		
	*/
	E get(int index);
	
	/**
	 * Insert passed arg "newElem" into position "index"
	 * Return previous (replaced) elem at "index"
	 * Valid "index" values are between 0 and "size - 1"
	 * If "index" is invalid, throws exception:
	 * "(set) Index %d is out of bounds"
	 * where %d is invalid index
	 * Example:
	 * 	given a dynamic list "foo" is [10, 20, 30, 40]
	 * 	then we call -- foo.set(2,888);
	 * Then, after "set" is finished, foo is: [10, 20, 888, 40]		
	 */
	E set(int index, E newElem);	
	
	//------------------------------------------
	// Common (Core)

	/** Returns nice display string for this QueryArray */
	String toString();	
	
	//------------------------------------------
	// Accessing (Other)	

	/**
	Return first element
	If list is empty, throws this runtime exception:
		"(first) Attempted to access element in empty list"
	*/
	E first();

	/**
	Return last element
	If list is empty, throws this runtime exception:
		"(last) Attempted to access element in empty list"
	*/
	E last();
	
	/**
	 * Returns a new QueryArrayIdea containing the elements of this list
	 * between the given index start (inclusive) and
	 * the given index stop (exclusive).
 	 * If either index is invalid, throws an IndexOutOfBoundsException with this text:
	 * 	  "(subList) index %d is out of bounds"
	 * where %d is replaced with the invalid index. 
	 */
	QueryArrayIdea<E> subList(int start, int stop);	
	
	
	
	//------------------------------------------
	// Searching	

	/** Returns the index of the first match, or -1 if no match */
	int findFirst(Function<E, Boolean> matchFct);
	
	//------------------------------------------
	// Convenience	

	/** Prints all elements to console, with newline after each */
	void printAll();

	/** Returns new QueryArrayIdea where mapFct is used to generate
	 *  new elements from existing elements in "this" object */
	<T> QueryArrayIdea<T> map(Function<E, T> mapFct);	

	/** Iterates over elements in "this" object. For each element,
	 * performs actionFct (passing element being iterated on) */
	void forEach(Consumer<E> actionFct);
	
	/**
	Returns a "native" list containing all elements from "this" object (in order).
	In Java, a "native" list type is "List".
	*/
	List<E> toNativeList();	

}