package model.queryarray;

import java.util.Arrays;
import java.util.List;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Supplier;

import model.linearpub.QueryArrayIdea;

/**
 * The QueryArray class is a simple shape class that
 * models a read-only queryable array.
 *
 * It provides convenience array query functions like
 * forEach, map, filter, ...
 
 * */

public class QueryArray<E> implements QueryArrayIdea<E> {

	//------------------------------------------
	// Object Structure (Instance Variables)	
	
	private final E[] elements;

	//------------------------------------------
	// Constructing - public	
	
	/** Returns new QueryArray object with passed name,
	 * country, lat and long values */
	public static <T> QueryArrayIdea<T> from(T[] elements) {
		return new QueryArray<>(elements);
	}
	
	/** Returns new QueryArray object of specified size (all elements are null)
	 *  NOTE WELL -- if you use this constructor make sure you fill all slots
	 *  up with objects using "set" -- we don't want to leave any null elements in array */
	public static <T> QueryArrayIdea<T> newWithSize(int sz) {
		@SuppressWarnings("unchecked")
		T[] newElements = (T[]) new Object[sz];		
		return QueryArray.from(newElements);
	}	

	//------------------------------------------
	// Constructing - private

	/** Construct new QueryArray from passed elements */	
	private QueryArray(E[] elements) {
		this.elements = elements;
	}

	//------------------------------------------
	// Instance Methods - Provided (Completed) Code
	
	@Override
	/**
	Returns a "native" list containing all elements from "this" object (in order).
	In Java, a "native" list type is "List".
	*/	
	public List<E> toNativeList() {
		return Arrays.asList(this.elements);
	}

	//------------------------------------------
	// Instance Methods
	
	//TODO	
	
}
