package test.testutil;

public class Employee {
	
	//=====================================================
	
	//Convenience methods for testing
	
	public static Employee[] newFour() {
		Employee[] emps = new Employee[4];
		emps[0] = Employee.fromIdName(1, "Asha");
		emps[1] = Employee.fromIdName(2, "Kofi");
		emps[2] = Employee.fromIdName(3, "Chin");
		emps[3] = Employee.fromIdName(4, "Riya");
		return emps;
	}
	
	//=============s========================================
	
	private final int id;
	private String name;
	
	/** Public constructor -- Returns new Employee given id and name */
	public static Employee fromIdName(int id, String name) {
		return new Employee(id, name);
	}

	/** Private constructor -- on instance side -- initialize ivars */	
	private Employee(int id, String name) {
		super();
		this.id = id;
		this.name = name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public int getId() {
		return id;
	}

	public String getName() {
		return name;
	}

	@Override
	public String toString() {
		return this.getName() + " (" + this.getId() + ")";
	}
	
	//=====================================================
	
	@Override	
	public int hashCode()  {
		int hashCode = 7;
		hashCode = 31 * hashCode + Integer.valueOf(this.id).hashCode();		
		hashCode = 31 * hashCode + (getName()!=null ? getName().hashCode() : 0);
		return hashCode;
	}	

	@Override
	public boolean equals(Object o) {
		if (o == this)
			return true;
		if (!(o instanceof Employee))
			return false;
		Employee other = (Employee)o;
		return
			this.getId() == other.getId()
				&& safeEquals(this.getName(), other.getName());
	}
	
	public boolean safeEquals(Object a, Object b)  {
		//If only one is nil, then false, use XOR*/
		if (a==null ^ b==null)
			return false;
		//We now know both are null, or not null, check for condition #1 first*/
		//If either is null, they both are
		if (a==null)
			return true;
		//Finally a safe equals
		return a.equals(b);
	}		

	
	
	
}